From c74da4072871f0883bd340ff679f65b627bbaa08 Mon Sep 17 00:00:00 2001
From: Jarmo Torvinen <jarmo.torvinen@jutel.fi>
Date: Wed, 29 Apr 2009 08:37:10 +0300
Subject: [PATCH] Initial implementation of Raw audio plugin

---
 modules/demux/Modules.am |    2 +
 modules/demux/rawaud.c   |  240 ++++++++++++++++++++++++++++++++++++++++++++++
 2 files changed, 242 insertions(+), 0 deletions(-)
 create mode 100644 modules/demux/rawaud.c

diff --git a/modules/demux/Modules.am b/modules/demux/Modules.am
index 97c6d04..7d206fe 100644
--- a/modules/demux/Modules.am
+++ b/modules/demux/Modules.am
@@ -5,6 +5,7 @@ SOURCES_demuxdump = demuxdump.c
 SOURCES_rawdv = rawdv.c
 SOURCES_rawvid = rawvid.c
 SOURCES_au = au.c
+SOURCES_rawaud = rawaud.c
 SOURCES_wav = wav.c
 SOURCES_live555 = live555.cpp ../access/mms/asf.c ../access/mms/buffer.c
 SOURCES_nsv = nsv.c
@@ -33,6 +34,7 @@ SOURCES_smf = smf.c
 libvlc_LTLIBRARIES += \
 	libaiff_plugin.la \
 	libau_plugin.la \
+	librawaud_plugin.la \
 	libdemux_cdg_plugin.la \
 	libdemuxdump_plugin.la \
 	libflacsys_plugin.la \
diff --git a/modules/demux/rawaud.c b/modules/demux/rawaud.c
new file mode 100644
index 0000000..3aa95fa
--- /dev/null
+++ b/modules/demux/rawaud.c
@@ -0,0 +1,240 @@
+/*****************************************************************************
+ * rawaud.c : raw audio input module for vlc
+ *****************************************************************************
+ * Copyright (C) 2009 the VideoLAN team
+ * $Id$
+ *
+ * Authors: Jarmo Torvinen <jarmo.torvinen@jutel.fi>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+/*****************************************************************************
+ * Preamble
+ *****************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_demux.h>
+
+/*****************************************************************************
+ * Module descriptor
+ *****************************************************************************/
+static int  Open ( vlc_object_t * );
+static void Close( vlc_object_t * );
+
+
+#define SAMPLERATE_TEXT N_("Audio samplerate (Hz)")
+#define SAMPLERATE_LONGTEXT N_("Audio sample rate in Hertz. Default is 48000 Hz.")
+
+#define CHANNELS_TEXT N_("Audio channels")
+#define CHANNELS_LONGTEXT N_("Audio channels in input stream. Numeric value >0. Default is 2.")
+
+#define FOURCC_TEXT N_("FOURCC code of raw input format")
+#define FOURCC_LONGTEXT N_( \
+    "FOURCC code of the raw input format. This is a four character string. Default is s16l." )
+
+#define LANG_TEXT N_("Forces the audio language.")
+#define LANG_LONGTEXT N_("Forces the audio language for the output mux. Three letter ISO639 code. Default is 'eng'. ")
+
+vlc_module_begin();
+    set_shortname( "Raw Audio" );
+    set_description( N_("Raw audio demuxer") );
+    set_capability( "demux", 0 );
+    set_category( CAT_INPUT );
+    set_subcategory( SUBCAT_INPUT_DEMUX );
+    set_callbacks( Open, Close );
+    add_shortcut( "rawaud" );
+    add_integer( "rawaud-channels", 2, 0, CHANNELS_TEXT, CHANNELS_LONGTEXT, false );
+    add_integer( "rawaud-samplerate", 48000, 0, SAMPLERATE_TEXT, SAMPLERATE_LONGTEXT, false );
+    add_string( "rawaud-fourcc", "s16l", NULL, FOURCC_TEXT, FOURCC_LONGTEXT, false );
+    add_string( "rawaud-lang", "eng", NULL, LANG_TEXT, LANG_LONGTEXT, false);
+vlc_module_end();
+
+/*****************************************************************************
+ * Definitions of structures used by this plugin
+ *****************************************************************************/
+struct demux_sys_t
+{
+    es_out_id_t *es;
+    es_format_t  fmt;
+    mtime_t      i_time;
+    int          i_frame_size;
+    mtime_t      i_frame_length;
+};
+
+
+/*****************************************************************************
+ * Local prototypes
+ *****************************************************************************/
+static int Demux( demux_t * );
+static int Control( demux_t *, int i_query, va_list args );
+
+
+/*****************************************************************************
+ * Open: initializes raw audio demuxer
+ *****************************************************************************/
+static int Open( vlc_object_t * p_this )
+{
+    demux_t     *p_demux = (demux_t*)p_this;
+    demux_sys_t *p_sys;
+    int i_samples;
+    
+    /* Set p_input field */
+    p_demux->pf_demux   = Demux;
+    p_demux->pf_control = Control;
+    p_demux->p_sys      = p_sys = malloc( sizeof( demux_sys_t ) );
+    if( !p_sys )
+        return VLC_ENOMEM;
+
+
+    es_format_Init( &p_sys->fmt, AUDIO_ES, 0 );
+
+    const char *psz_fourcc = var_CreateGetString( p_demux, "rawaud-fourcc" );
+
+    p_sys->fmt.i_codec = VLC_FOURCC( psz_fourcc[0], psz_fourcc[1], psz_fourcc[2], psz_fourcc[3] ); 
+
+    free(psz_fourcc);
+    // get the bits per sample ratio based on codec
+    switch (p_sys->fmt.i_codec)
+    {
+
+        case VLC_FOURCC( 'f', 'l', '6', '4' ):
+            p_sys->fmt.audio.i_bitspersample = 64;
+            break;
+        
+        case VLC_FOURCC( 'f', 'l', '3', '2' ):
+        case VLC_FOURCC( 's', '3', '2', 'l' ):
+        case VLC_FOURCC( 's', '3', '2', 'b' ):
+        case VLC_FOURCC( 'i', 'n', '3', '2' ):
+            p_sys->fmt.audio.i_bitspersample = 32;
+            break;
+
+        case VLC_FOURCC( 's', '2', '4', 'l' ):
+        case VLC_FOURCC( 's', '2', '4', 'b' ):
+        case VLC_FOURCC( 'i', 'n', '2', '4' ):
+        case VLC_FOURCC( '4', '2', 'n', 'i' ):
+            p_sys->fmt.audio.i_bitspersample = 24;
+            break;
+
+        case VLC_FOURCC( 's', '1', '6', 'l' ):
+        case VLC_FOURCC( 's', '1', '6', 'b' ):
+            p_sys->fmt.audio.i_bitspersample = 16;
+            break;
+
+        case VLC_FOURCC( 's', '8', ' ', ' ' ):
+        case VLC_FOURCC( 'u', '8', ' ', ' ' ):
+            p_sys->fmt.audio.i_bitspersample = 8;
+            break;
+
+        default:
+            
+            msg_Err( p_demux, "unknown fourcc format %4.4s", (char*)&p_sys->fmt.i_codec);
+        return VLC_EGENERIC;
+
+    }
+
+
+    p_sys->fmt.psz_language = var_CreateGetString( p_demux, "rawaud-lang" );
+    p_sys->fmt.audio.i_channels = var_CreateGetInteger( p_demux, "rawaud-channels" );
+    p_sys->fmt.audio.i_rate = var_CreateGetInteger( p_demux, "rawaud-samplerate" );
+
+    msg_Dbg( p_demux, "got parameters: channels=%d , samplerate=%d, fourcc=%4.4s, bits per sample = %d", 
+        p_sys->fmt.audio.i_channels, p_sys->fmt.audio.i_rate, (char*)&p_sys->fmt.i_codec, 
+        p_sys->fmt.audio.i_bitspersample);
+
+    msg_Dbg( p_demux, "format initialized");
+
+    p_sys->fmt.i_bitrate = p_sys->fmt.audio.i_rate *
+                           p_sys->fmt.audio.i_channels *
+                           p_sys->fmt.audio.i_bitspersample;
+
+    msg_Dbg( p_demux, "bitrate is %d bit/s", p_sys->fmt.i_bitrate);
+
+    /* add the es */
+    p_sys->es = es_out_Add( p_demux->out, &p_sys->fmt );
+
+    msg_Dbg( p_demux, "elementary stream added");
+
+    /* calculate 50ms frame size/time */
+    i_samples = __MAX( p_sys->fmt.audio.i_rate / 20, 1 );
+
+    p_sys->i_frame_size = i_samples * p_sys->fmt.audio.i_channels *
+                          ( (p_sys->fmt.audio.i_bitspersample + 7) / 8 );
+
+    msg_Dbg( p_demux, "frame size is %d bytes ", p_sys->i_frame_size);
+
+    p_sys->i_frame_length = (mtime_t)1000000 *
+                            (mtime_t)i_samples /
+                            (mtime_t)p_sys->fmt.audio.i_rate;
+
+    msg_Dbg( p_demux, "frame length is %d us", p_sys->i_frame_length);
+
+    return VLC_SUCCESS;
+}
+
+/*****************************************************************************
+ * Close: frees unused data
+ *****************************************************************************/
+static void Close( vlc_object_t *p_this )
+{
+    demux_t     *p_demux = (demux_t*)p_this;
+    demux_sys_t *p_sys  = p_demux->p_sys;
+
+    free( p_sys->fmt.psz_language);
+    free( p_sys );
+}
+
+/*****************************************************************************
+ * Demux: reads and demuxes data packets
+ *****************************************************************************
+ * Returns -1 in case of error, 0 in case of EOF, 1 otherwise
+ *****************************************************************************/
+static int Demux( demux_t *p_demux )
+{
+    demux_sys_t *p_sys  = p_demux->p_sys;
+    block_t     *p_block;
+
+    /* Call the pace control */
+    es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_sys->i_time );
+
+    if( ( p_block = stream_Block( p_demux->s, p_sys->i_frame_size ) ) == NULL )
+    {
+        /* EOF */
+        return 0;
+    }
+
+    p_block->i_dts = p_block->i_pts = p_sys->i_time;
+    es_out_Send( p_demux->out, p_sys->es, p_block );
+
+    p_sys->i_time += p_sys->i_frame_length;
+
+    return 1;
+}
+
+/*****************************************************************************
+ * Control:
+ *****************************************************************************/
+static int Control( demux_t *p_demux, int i_query, va_list args )
+{
+    demux_sys_t *p_sys  = p_demux->p_sys;
+
+    return demux_vaControlHelper( p_demux->s, 0, -1,
+                                   p_sys->fmt.i_bitrate, 1, i_query, args );
+}
-- 
1.6.0.4

